@@ -28,16 +28,16 @@ __license__   = "LGPL"
 import pyatspi
 from gi.repository import Atspi, Atk
 
-import braille
-import generator
-import orca
-import orca_state
-import settings
+from . import braille
+from . import debug
+from . import generator
+from . import messages
+from . import orca_state
+from . import settings
+from . import settings_manager
+from .braille_rolenames import shortRoleNames
 
-from braille_rolenames import shortRoleNames
-from orca_i18n import ngettext  # for ngettext support
-
-_settingsManager = getattr(orca, '_settingsManager')
+_settingsManager = settings_manager.getManager()
 
 class Space:
     """A dummy class to indicate we want to insert a space into an
@@ -73,6 +73,11 @@ class BrailleGenerator(generator.Generator):
         globalsDict['asString'] = self.asString
 
     def generateBraille(self, obj, **args):
+        if not _settingsManager.getSetting('enableBraille') \
+           and not _settingsManager.getSetting('enableBrailleMonitor'):
+            debug.println(debug.LEVEL_INFO, "BRAILLE: generation disabled")
+            return [[], None]
+
         if obj == orca_state.locusOfFocus \
            and not args.get('formatType', None):
             args['formatType'] = 'focused'
@@ -127,11 +132,23 @@ class BrailleGenerator(generator.Generator):
         """
         result = []
         role = args.get('role', obj.getRole())
+        verbosityLevel = _settingsManager.getSetting('brailleVerbosityLevel')
+
+        doNotPresent = [pyatspi.ROLE_UNKNOWN,
+                        pyatspi.ROLE_FILLER,
+                        pyatspi.ROLE_EXTENDED]
+
+        # egg-list-box, e.g. privacy panel in gnome-control-center
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+            doNotPresent.append(obj.getRole())
+
+        if verbosityLevel == settings.VERBOSITY_LEVEL_BRIEF:
+            doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
+
         if (role in _settingsManager.getSetting('brailleForceRoles'))\
-           or ((_settingsManager.getSetting('brailleVerbosityLevel') \
-                == settings.VERBOSITY_LEVEL_VERBOSE)\
+           or (verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE\
                and not args.get('readingRow', False)\
-               and (role != pyatspi.ROLE_UNKNOWN)):
+               and role not in doNotPresent):
             result.append(self.getLocalizedRoleName(obj, role))
         return result
 
@@ -166,6 +183,15 @@ class BrailleGenerator(generator.Generator):
 
         return Atk.role_get_localized_name(atkRole)
 
+    def _generateUnrelatedLabels(self, obj, **args):
+        result = []
+        labels = self._script.utilities.unrelatedLabels(obj)
+        for label in labels:
+            name = self._generateName(label, **args)
+            result.extend(name)
+
+        return result
+
     #####################################################################
     #                                                                   #
     # Keyboard shortcut information                                     #
@@ -202,13 +228,8 @@ class BrailleGenerator(generator.Generator):
         except:
             alertAndDialogCount = 0
         if alertAndDialogCount > 0:
-            # Translators: this tells the user how many unfocused
-            # alert and dialog windows plus the total number of
-            # windows that this application has.
-            #
-            result.append(ngettext("(%d dialog)",
-                                   "(%d dialogs)",
-                                   alertAndDialogCount) % alertAndDialogCount)
+             result.append(messages.dialogCountBraille(alertAndDialogCount))
+
         return result
 
     def _generateAncestors(self, obj, **args):
@@ -281,6 +302,29 @@ class BrailleGenerator(generator.Generator):
         result.reverse()
         return result
 
+    def _generateFocusedItem(self, obj, **args):
+        result = []
+        role = args.get('role', obj.getRole())
+        if role != pyatspi.ROLE_LIST:
+            return result
+
+        try:
+            s = obj.querySelection()
+        except NotImplementedError:
+            isFocused = \
+                lambda x: x and x.getState().contains(pyatspi.STATE_FOCUSED)
+            items = pyatspi.utils.findAllDescendants(obj, isFocused)
+        else:
+            items = [s.getSelectedChild(i) for i in range(s.nSelectedChildren)]
+            if not items and obj.childCount:
+                items.append(obj[0])
+
+        items = list(map(self._generateName, items))
+        for item in items:
+            result.extend(item)
+
+        return result
+
     #####################################################################
     #                                                                   #
     # Unfortunate hacks.                                                #
@@ -388,7 +432,7 @@ class BrailleGenerator(generator.Generator):
     def asString(self, content, delimiter=" "):
         combined = ""
         prior = None
-        if isinstance(content, basestring):
+        if isinstance(content, str):
             combined = content
         elif content and isinstance(content, list):
             # Strip off leading and trailing spaces.
@@ -398,11 +442,6 @@ class BrailleGenerator(generator.Generator):
             while content and isinstance(content[-1], Space):
                 content = content[0:-1]
             for element in content:
-                try:
-                    element = element.decode("UTF-8")
-                except UnicodeEncodeError:
-                    pass
-
                 if isinstance(element, Space) and prior:
                     combined += element.delimiter
                     prior = None
